@@ -42,7 +42,8 @@ def test_check_version():
 def test_install_tool(tmp_path, monkeypatch):
     dname = tmp_path / "install_tool"
     monkeypatch.setenv("PATHTOOLS", str(dname))
+    Cnt = {"DIRTOOLS": "NiftyPET_tools_test"}
     assert not path.exists(dname)
-    with pytest.raises(UnboundLocalError):
-        tls.install_tool("", {})
-    assert path.exists(dname)
+    with pytest.raises(ValueError):
+        tls.install_tool("", Cnt)
+    assert path.exists(dname / Cnt["DIRTOOLS"])
